@@ -25,6 +25,7 @@ GLib = modules['GLib']._introspection_module
 
 __all__ = []
 
+
 class _VariantCreator(object):
 
     _LEAF_CONSTRUCTORS = {
@@ -83,7 +84,7 @@ class _VariantCreator(object):
     def _create_tuple(self, format, args):
         '''Handle the case where the outermost type of format is a tuple.'''
 
-        format = format[1:] # eat the '('
+        format = format[1:]  # eat the '('
         builder = GLib.VariantBuilder.new(variant_type_from_string('r'))
         if args is not None:
             if not args or type(args[0]) != type(()):
@@ -109,7 +110,7 @@ class _VariantCreator(object):
             rest_format = self._create(rest_format, None)[1]
             if not rest_format.startswith('}'):
                 raise ValueError('dictionary type string not closed with }')
-            rest_format = rest_format[1:] # eat the }
+            rest_format = rest_format[1:]  # eat the}
             element_type = format[:len(format) - len(rest_format)]
             builder = GLib.VariantBuilder.new(variant_type_from_string(element_type))
         else:
@@ -120,7 +121,7 @@ class _VariantCreator(object):
 
                 if not rest_format.startswith('}'):
                     raise ValueError('dictionary type string not closed with }')
-                rest_format = rest_format[1:] # eat the }
+                rest_format = rest_format[1:]  # eat the}
 
                 entry = GLib.VariantBuilder.new(variant_type_from_string('{?*}'))
                 entry.add_value(key_v)
@@ -150,17 +151,18 @@ class _VariantCreator(object):
             args = args[1:]
         return (builder.end(), rest_format, args)
 
+
 class Variant(GLib.Variant):
     def __new__(cls, format_string, value):
         '''Create a GVariant from a native Python object.
 
         format_string is a standard GVariant type signature, value is a Python
         object whose structure has to match the signature.
-        
+
         Examples:
           GLib.Variant('i', 1)
           GLib.Variant('(is)', (1, 'hello'))
-          GLib.Variant('(asa{sv})', ([], {'foo': GLib.Variant('b', True), 
+          GLib.Variant('(asa{sv})', ([], {'foo': GLib.Variant('b', True),
                                           'bar': GLib.Variant('i', 2)}))
         '''
         creator = _VariantCreator()
@@ -169,6 +171,9 @@ class Variant(GLib.Variant):
             raise TypeError('invalid remaining format string: "%s"' % rest_format)
         return v
 
+    def __del__(self):
+        self.unref()
+
     def __repr__(self):
         return '<GLib.Variant(%s)>' % getattr(self, 'print')(True)
 
@@ -205,8 +210,8 @@ class Variant(GLib.Variant):
             'h': self.get_handle,
             'd': self.get_double,
             's': self.get_string,
-            'o': self.get_string, # object path
-            'g': self.get_string, # signature
+            'o': self.get_string,  # object path
+            'g': self.get_string,  # signature
         }
 
         # simple values
@@ -216,7 +221,7 @@ class Variant(GLib.Variant):
 
         # tuple
         if self.get_type_string().startswith('('):
-            res = [self.get_child_value(i).unpack() 
+            res = [self.get_child_value(i).unpack()
                     for i in range(self.n_children())]
             return tuple(res)
 
@@ -230,7 +235,7 @@ class Variant(GLib.Variant):
 
         # array
         if self.get_type_string().startswith('a'):
-            return [self.get_child_value(i).unpack() 
+            return [self.get_child_value(i).unpack()
                     for i in range(self.n_children())]
 
         # variant (just unbox transparently)
@@ -245,7 +250,7 @@ class Variant(GLib.Variant):
 
         If the signature is not a tuple, it returns one element with the entire
         signature. If the signature is an empty tuple, the result is [].
-        
+
         This is useful for e. g. iterating over method parameters which are
         passed as a single Variant.
         '''
@@ -257,7 +262,7 @@ class Variant(GLib.Variant):
 
         result = []
         head = ''
-        tail = signature[1:-1] # eat the surrounding ( )
+        tail = signature[1:-1]  # eat the surrounding ()
         while tail:
             c = tail[0]
             head += c
@@ -292,6 +297,7 @@ class Variant(GLib.Variant):
     #
     # Pythonic iterators
     #
+
     def __len__(self):
         if self.get_type_string() in ['s', 'o', 'g']:
             return len(self.get_string())
@@ -336,6 +342,7 @@ class Variant(GLib.Variant):
     #
     # Pythonic bool operations
     #
+
     def __nonzero__(self):
         return self.__bool__()
 
@@ -365,10 +372,12 @@ class Variant(GLib.Variant):
             res.append(v.get_child_value(0).unpack())
         return res
 
+
 @classmethod
 def new_tuple(cls, *elements):
     return variant_new_tuple(elements)
 
+
 def get_string(self):
     value, length = GLib.Variant.get_string(self)
     return value
@@ -377,4 +386,3 @@ setattr(Variant, 'new_tuple', new_tuple)
 setattr(Variant, 'get_string', get_string)
 
 __all__.append('Variant')
-
