@@ -71,9 +71,8 @@ def _install_enums(module, dest=None, strip=''):
         try:
             if issubclass(obj, GObject.GFlags):
                 for value, flag in obj.__flags_values__.items():
-                    for name in flag.value_names:
-                        name = name.replace(modname + '_', '')
-                        setattr(dest, name, flag)
+                    name = flag.value_names[-1].replace(modname + '_', '')
+                    setattr(dest, name, flag)
         except TypeError:
             continue
 
@@ -135,6 +134,26 @@ def enable_gtk(version='2.0'):
     Gdk.pixbuf_new_from_file = GdkPixbuf.Pixbuf.new_from_file
     Gdk.PixbufLoader = GdkPixbuf.PixbufLoader.new_with_type
 
+    orig_get_formats = GdkPixbuf.Pixbuf.get_formats
+
+    def get_formats():
+        formats = orig_get_formats()
+        result = []
+
+        def make_dict(format_):
+            result = {}
+            result['description'] = format_.get_description()
+            result['name'] = format_.get_name()
+            result['mime_types'] = format_.get_mime_types()
+            result['extensions'] = format_.get_extensions()
+            return result
+
+        for format_ in formats:
+            result.append(make_dict(format_))
+        return result
+
+    Gdk.pixbuf_get_formats = get_formats
+
     orig_get_frame_extents = Gdk.Window.get_frame_extents
 
     def get_frame_extents(window):
@@ -155,6 +174,9 @@ def enable_gtk(version='2.0'):
         return orig_get_origin(self)[1:]
     Gdk.Window.get_origin = get_origin
 
+    Gdk.screen_width = Gdk.Screen.width
+    Gdk.screen_height = Gdk.Screen.height
+
     # gtk
     gi.require_version('Gtk', version)
     from gi.repository import Gtk
@@ -309,8 +331,19 @@ def enable_gtk(version='2.0'):
     Gtk.icon_theme_get_default = Gtk.IconTheme.get_default
     Gtk.image_new_from_pixbuf = Gtk.Image.new_from_pixbuf
     Gtk.image_new_from_stock = Gtk.Image.new_from_stock
+    Gtk.image_new_from_animation = Gtk.Image.new_from_animation
+    Gtk.image_new_from_icon_set = Gtk.Image.new_from_icon_set
+    Gtk.image_new_from_file = Gtk.Image.new_from_file
     Gtk.settings_get_default = Gtk.Settings.get_default
     Gtk.window_set_default_icon = Gtk.Window.set_default_icon
+    Gtk.clipboard_get = Gtk.Clipboard.get
+
+    #AccelGroup
+    Gtk.AccelGroup.connect_group = Gtk.AccelGroup.connect
+
+    #StatusIcon
+    Gtk.status_icon_position_menu = Gtk.StatusIcon.position_menu
+    Gtk.StatusIcon.set_tooltip = Gtk.StatusIcon.set_tooltip_text
 
     # Scale
 
@@ -351,9 +384,9 @@ def enable_gtk(version='2.0'):
 
         def __getitem__(self, state):
             color = self.context.get_background_color(state)
-            return Gdk.Color(red=color.red,
-                             green=color.green,
-                             blue=color.blue)
+            return Gdk.Color(red=int(color.red * 65535),
+                             green=int(color.green * 65535),
+                             blue=int(color.blue * 65535))
 
     class Styles(object):
         def __init__(self, widget):
