@@ -15,6 +15,14 @@ mydir = os.path.dirname(os.path.abspath(__file__))
 
 # force untranslated messages, as we check for them in some tests
 os.environ['LC_MESSAGES'] = 'C'
+os.environ['G_DEBUG'] = 'fatal-warnings fatal-criticals'
+
+# make Gio able to find our gschemas.compiled in tests/. This needs to be set
+# before importing Gio. Support a separate build tree, so look in build dir
+# first.
+os.environ['GSETTINGS_BACKEND'] = 'memory'
+os.environ['GSETTINGS_SCHEMA_DIR'] = os.environ.get('TESTS_BUILDDIR',
+        os.path.dirname(__file__))
 
 # Load tests.
 if 'TEST_NAMES' in os.environ:
@@ -40,4 +48,4 @@ suite = loader.loadTestsFromNames(names)
 runner = unittest.TextTestRunner(verbosity=2)
 result = runner.run(suite)
 if not result.wasSuccessful():
-    sys.exit(1) # exit code so "make check" reports error
+    sys.exit(1)  # exit code so "make check" reports error
