@@ -9,11 +9,16 @@ sys.path.insert(0, "../")
 from sys import getrefcount
 
 import copy
-import cairo
+try:
+    import cairo
+    has_cairo = True
+except ImportError:
+    has_cairo = False
 
 from gi.repository import GObject
 from gi.repository import GLib
 from gi.repository import Gio
+from gi.repository import Gtk
 from gi.repository import Regress as Everything
 
 if sys.version_info < (3, 0):
@@ -25,6 +30,7 @@ else:
 
 class TestEverything(unittest.TestCase):
 
+    @unittest.skipUnless(has_cairo, 'built without cairo support')
     def test_cairo_context(self):
         context = Everything.test_cairo_context_full_return()
         self.assertTrue(isinstance(context, cairo.Context))
@@ -33,6 +39,7 @@ class TestEverything(unittest.TestCase):
         context = cairo.Context(surface)
         Everything.test_cairo_context_none_in(context)
 
+    @unittest.skipUnless(has_cairo, 'built without cairo support')
     def test_cairo_surface(self):
         surface = Everything.test_cairo_surface_none_return()
         self.assertTrue(isinstance(surface, cairo.ImageSurface))
@@ -184,7 +191,7 @@ class TestEverything(unittest.TestCase):
         # https://bugzilla.gnome.org/show_bug.cgi?id=666636
         # workaround by explicitly building a GStrv object
         class GStrv(list):
-            __gtype__ = GObject.type_from_name('GStrv')
+            __gtype__ = GObject.TYPE_STRV
 
         data = {'integer': 12,
                 'boolean': True,
@@ -470,13 +477,12 @@ class TestClosures(unittest.TestCase):
         self.assertTrue(self.called)
         self.assertEqual(result, 43)
 
-    # https://bugzilla.gnome.org/show_bug.cgi?id=656554
-
-    @unittest.expectedFailure
     def test_variant(self):
         def callback(variant):
-            self.assertEqual(variant.get_type_string(), 'i')
             self.called = True
+            if variant is None:
+                return None
+            self.assertEqual(variant.get_type_string(), 'i')
             return GLib.Variant('i', variant.get_int32() + 1)
 
         self.called = False
@@ -485,6 +491,15 @@ class TestClosures(unittest.TestCase):
         self.assertEqual(result.get_type_string(), 'i')
         self.assertEqual(result.get_int32(), 43)
 
+        self.called = False
+        result = Everything.test_closure_variant(callback, None)
+        self.assertTrue(self.called)
+        self.assertEqual(result, None)
+
+        self.called = False
+        self.assertRaises(TypeError, Everything.test_closure_variant, callback, 'foo')
+        self.assertFalse(self.called)
+
 
 class TestProperties(unittest.TestCase):
 
@@ -499,12 +514,12 @@ class TestProperties(unittest.TestCase):
         self.assertEqual(object_.props.float, 0.0)
         object_.props.float = 42.42
         self.assertTrue(isinstance(object_.props.float, float))
-        self.assertAlmostEquals(object_.props.float, 42.42, places=5)
+        self.assertAlmostEqual(object_.props.float, 42.42, places=5)
 
         self.assertEqual(object_.props.double, 0.0)
         object_.props.double = 42.42
         self.assertTrue(isinstance(object_.props.double, float))
-        self.assertAlmostEquals(object_.props.double, 42.42, places=5)
+        self.assertAlmostEqual(object_.props.double, 42.42, places=5)
 
         self.assertEqual(object_.props.string, None)
         object_.props.string = 'mec'
@@ -542,6 +557,14 @@ class TestProperties(unittest.TestCase):
         self.assertTrue(isinstance(object_.props.boxed, Everything.TestBoxed))
         self.assertEqual(object_.props.boxed.some_int8, 42)
 
+    def test_boxed_equality(self):
+        boxed = Everything.TestBoxedC()
+        # TestBoxedC uses refcounting, so we know that
+        # the pointer is the same when copied
+        copy = boxed.copy()
+        self.assertEqual(boxed, copy)
+        self.assertNotEqual(id(boxed), id(copy))
+
     def test_gtype(self):
         object_ = Everything.TestObj()
         self.assertEqual(object_.props.gtype, GObject.TYPE_INVALID)
@@ -650,3 +673,10 @@ class TestSignals(unittest.TestCase):
 
         obj.connect('sig-with-obj', callback)
         obj.emit_sig_with_obj()
+
+
+class TestPango(unittest.TestCase):
+    def test_cairo_font_options(self):
+        screen = Gtk.Window().get_screen()
+        font_opts = screen.get_font_options()
+        self.assertEqual(type(font_opts.get_subpixel_order()), int)
