@@ -499,13 +499,26 @@ class TestGdk(unittest.TestCase):
         self.assertEqual(color.green, 200)
         self.assertEqual(color.blue, 300)
         self.assertEqual(color, Gdk.Color(100, 200, 300))
-        self.assertNotEquals(color, Gdk.Color(1, 2, 3))
+        self.assertNotEqual(color, Gdk.Color(1, 2, 3))
+
+    def test_color_floats(self):
+        self.assertEqual(Gdk.Color(13107, 21845, 65535),
+                         Gdk.Color.from_floats(0.2, 1.0 / 3.0, 1.0))
+
+        self.assertEqual(Gdk.Color(13107, 21845, 65535).to_floats(),
+                         (0.2, 1.0 / 3.0, 1.0))
+
+        self.assertEqual(Gdk.RGBA(0.2, 1.0 / 3.0, 1.0, 0.5).to_color(),
+                         Gdk.Color.from_floats(0.2, 1.0 / 3.0, 1.0))
+
+        self.assertEqual(Gdk.RGBA.from_color(Gdk.Color(13107, 21845, 65535)),
+                         Gdk.RGBA(0.2, 1.0 / 3.0, 1.0, 1.0))
 
     def test_rgba(self):
         self.assertEqual(Gdk.RGBA, overrides.Gdk.RGBA)
         rgba = Gdk.RGBA(0.1, 0.2, 0.3, 0.4)
         self.assertEqual(rgba, Gdk.RGBA(0.1, 0.2, 0.3, 0.4))
-        self.assertNotEquals(rgba, Gdk.RGBA(0.0, 0.2, 0.3, 0.4))
+        self.assertNotEqual(rgba, Gdk.RGBA(0.0, 0.2, 0.3, 0.4))
         self.assertEqual(rgba.red, 0.1)
         self.assertEqual(rgba.green, 0.2)
         self.assertEqual(rgba.blue, 0.3)
@@ -513,6 +526,10 @@ class TestGdk(unittest.TestCase):
         rgba.green = 0.9
         self.assertEqual(rgba.green, 0.9)
 
+        # Iterator/tuple convsersion
+        self.assertEqual(tuple(Gdk.RGBA(0.1, 0.2, 0.3, 0.4)),
+                         (0.1, 0.2, 0.3, 0.4))
+
     def test_event(self):
         event = Gdk.Event.new(Gdk.EventType.CONFIGURE)
         self.assertEqual(event.type, Gdk.EventType.CONFIGURE)
@@ -1251,7 +1268,7 @@ class TestGtk(unittest.TestCase):
                 return -1
             if v2.startswith('m') and not v1.startswith('m'):
                 return 1
-            return cmp(v1, v2)
+            return (v1 > v2) - (v1 < v2)
 
         list_store = Gtk.ListStore(int, str)
         list_store.set_sort_func(2, comp1, None)
@@ -1374,7 +1391,7 @@ class TestGtk(unittest.TestCase):
         for i in range(100):
             label = 'this is row #%d' % i
             parent = tree_store.append(None, (DerivedIntType(i), DerivedStrType(label),))
-            self.assertNotEquals(parent, None)
+            self.assertNotEqual(parent, None)
             for j in range(20):
                 label = 'this is child #%d of node #%d' % (j, i)
                 child = tree_store.append(parent, (j, label,))
@@ -1499,6 +1516,15 @@ class TestGtk(unittest.TestCase):
         self.assertRaises(ValueError, tree_store.get, aiter, 1, 100)
         self.assertEqual(tree_store.get(aiter, 0, 1), (10, 'this is row #10'))
 
+        # check __delitem__
+        self.assertEqual(len(tree_store), 100)
+        aiter = tree_store.get_iter(10)
+        del tree_store[aiter]
+        self.assertEqual(len(tree_store), 99)
+        self.assertRaises(TypeError, tree_store.__delitem__, None)
+        self.assertRaises(IndexError, tree_store.__delitem__, -101)
+        self.assertRaises(IndexError, tree_store.__delitem__, 101)
+
     def test_tree_model_edit(self):
         model = Gtk.ListStore(int, str, float)
         model.append([1, "one", -0.1])
@@ -1866,8 +1892,8 @@ class TestGtk(unittest.TestCase):
 
         hscrollbar = Gtk.HScrollbar()
         vscrollbar = Gtk.VScrollbar()
-        self.assertNotEquals(hscrollbar.props.adjustment, adjustment)
-        self.assertNotEquals(vscrollbar.props.adjustment, adjustment)
+        self.assertNotEqual(hscrollbar.props.adjustment, adjustment)
+        self.assertNotEqual(vscrollbar.props.adjustment, adjustment)
 
         hscrollbar = Gtk.HScrollbar(adjustment)
         vscrollbar = Gtk.VScrollbar(adjustment)
